/*
 * PrettyTime is an OpenSource Java time comparison library for creating human
 * readable time.
 * 
 * Copyright (C) 2009 - Lincoln Baxter, III <lincoln@ocpsoft.com>
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see the file COPYING.LESSER3 or visit the
 * GNU website at <http://www.gnu.org/licenses/>.
 */
package com.ocpsoft.pretty.time;

import com.ocpsoft.pretty.time.units.Century;
import com.ocpsoft.pretty.time.units.Day;
import com.ocpsoft.pretty.time.units.Decade;
import com.ocpsoft.pretty.time.units.Hour;
import com.ocpsoft.pretty.time.units.JustNow;
import com.ocpsoft.pretty.time.units.Millennium;
import com.ocpsoft.pretty.time.units.Millisecond;
import com.ocpsoft.pretty.time.units.Minute;
import com.ocpsoft.pretty.time.units.Month;
import com.ocpsoft.pretty.time.units.Second;
import com.ocpsoft.pretty.time.units.Week;
import com.ocpsoft.pretty.time.units.Year;

/**
 * Defines a Unit of time (e.g. seconds, minutes, hours) and its conversion to
 * milliseconds.
 * 
 * @author lb3
 */
public interface TimeUnit
{
    public static final TimeUnit MILLISECOND = new Millisecond();
    public static final TimeUnit SECOND = new Second();
    public static final TimeUnit MINUTE = new Minute();
    public static final TimeUnit HOUR = new Hour();
    public static final TimeUnit MONTH = new Month();
    public static final TimeUnit DAY = new Day();
    public static final TimeUnit JUST_NOW = new JustNow();
    public static final TimeUnit WEEK = new Week();
    public static final TimeUnit YEAR = new Year();
    public static final TimeUnit DECADE = new Decade();
    public static final TimeUnit CENTURY = new Century();
    public static final TimeUnit MILLENNIUM = new Millennium();

    /**
     * The number of milliseconds represented by each instance of this TimeUnit.
     * Must be a positive number greater than zero.
     * 
     * @return
     */
    public long getMillisPerUnit();

    /**
     * The maximum quantity of this Unit to be used as a threshold for the next
     * largest Unit (e.g. if one <code>Second</code> represents 1000ms, and
     * <code>Second</code> has a maxQuantity of 5, then if the difference
     * between compared timestamps is larger than 5000ms, PrettyTime will move
     * on to the next smallest TimeUnit for calculation; <code>Minute</code>, by
     * default)
     * <p>
     * millisPerUnit * maxQuantity = maxAllowedMs
     * <p>
     * If maxQuantity is zero, it will be equal to the next highest
     * <code>TimeUnit.getMillisPerUnit() /
     * this.getMillisPerUnit()</code> or infinity if there are no greater
     * TimeUnits
     * 
     * @return
     */
    public long getMaxQuantity();

    /**
     * The grammatically singular name for this unit of time. (e.g. one
     * "second")
     * 
     * @return
     */
    public String getName();

    /**
     * The grammatically plural name for this unit of time. (e.g. many
     * "seconds")
     * 
     * @return
     */
    public String getPluralName();

    /**
     * The current {@link BasicTimeFormat} for this unit of time.
     * 
     * @return
     */
    public TimeFormat getFormat();
}
